//
// Slideshow/animation program. View picture files one by one or
// cycle through them
//
// Because of CLIP's picture model, you can't change the size of a displayed
// picture. This means that when different-sized pictures are shown in
// sequence a window has to be destroyed and another created. To avoid the nasty
// flashes caused by this, a "smooth" slideshow program would either rescale 
// pictures in the sequence or use showat() functions.
//
// John Robinson 1999, 2005.

#include "picture.h"
int main(int argc, char *argv[]) {
	if (argc < 2) {
		cerr << "Usage: diss pic1 pic2 ..." << endl;
		exit(1);
		}
	double framerate = 0.0;
	int interval = 1000;	
	colour_picture *disp = new colour_picture(argv[1]);
	colour_picture *next = 0;
	disp->show(argv[1]);
	parampic frpic(-2,2,framerate,"10^n frames/s");
	int currpic = 1;
	if (argc > 2)
		currpic++;
	int midclickleft = 0;
	int pausing = 0;
	int countdown = interval/10;
	while (!disp->closerequested()&&!disp->pointrightbutton()) {
	    if (midclickleft) {	// Currently clicking inside a picture
		if (!disp->pointleftbutton()) {
			if (pausing) {
				next = new colour_picture(argv[currpic]);
				if ((next->nrows()==disp->nrows())&&
						(next->ncols()==disp->ncols())&&
						(next->ismonochrome()==disp->ismonochrome())){
					*disp = *next;
					delete next;
	    				disp->fastreshow(argv[currpic]);
					}
				else {
					disp->unshow();
					delete disp;
					disp = next;
	    				disp->show(argv[currpic]);
					}
	    			currpic++;
	    			if (currpic >= argc)
					currpic = 1;
				}
			midclickleft = 0;
			}
		}
	    else if (disp->pointleftbutton()) {
		midclickleft = 1;
		pausing = 1;
		}
	    if (frpic.X(framerate)) {
		pausing = 0;
		interval = (int)(1000.0/pow(10,framerate));
		countdown = 1;	// So will start immediately with new interval
		}
	    if (midclickleft || pausing)
		continue;
	    disp->wait(10);
	    if (--countdown != 0)
		continue;
	    countdown = interval/10;
	    next = new colour_picture(argv[currpic]);
	    if ((next->nrows()==disp->nrows())&&
				(next->ncols()==disp->ncols())&&
				(next->ismonochrome()==disp->ismonochrome())){
		*disp = *next;
		delete next;
	    	disp->fastreshow(argv[currpic]);
		}
	    else {
		disp->unshow();
		delete disp;
		disp = next;
	    	disp->show(argv[currpic]);
		}
	    currpic++;
	    if (currpic >= argc)
		currpic = 1;
	    }
	disp->unshow();
	delete disp;
	return 0;
	}
