//
// Motion estimation based on closest-pel full-search block matching
//
// Usage: motionblocks previous_frame current_frame blocksize searchrange m
//
// where blocksize is usually 16 (for 16 x 16 square blocks), search region
// is also square and searchrange has to be odd, because the region is
// centred on the current position, and m is the method to use. If m is 'a'
// absolute differences will be used, if m is 's' squared differences.
//
// Writes out text specifying motion vectors (mv's) in the format:
//	Line 1:		blocksize	maximum_mv_displacement
//	Other lines:	y_coord_of_block_centre x_coord_of_block_centre
//				y_component_of_mv x_component_of_mv
//			(i.e. all four entries on each line)
//
// John Robinson
//
#include "usage.h"
#include "picture.h"
#include <stdlib.h>
#include <math.h>

int main(int argc, char *argv[])
	{
	usage("motionblocks previous current blocksize searchrange method");
	picture_of_int prev(argv[1]);
	picture_of_int in(argv[2]);
	int blocksize = atoi(argv[3]);
	int searchrange = atoi(argv[4]);
	if ((searchrange/2)*2 == searchrange) {
		cerr << "searchrange should be odd - centred on (0,0)\n";
		exit(1);
		}
	searchrange /= 2;	//Now one-sided range
	char method = argv[5][0];
	if ((method != 's')&&(method != 'a')) {
		cerr << "method must be s[quared error] or a[bsolute error]\n"; 
		exit(1);
		}
	prev.dc_pad(); // Extend edge values out beyond picture borders
	int ymax = in.nrows();
	int xmax = in.ncols();
	irange xblock(0,1,blocksize-1);
	irange yblock(0,1,blocksize-1);
	picture_of_int matches(2*searchrange+1, 2*searchrange+1);
	// Write out blocksize and maximum displacement
	cout << blocksize << " " << searchrange << endl;
	for (int i = 0; i < ymax; i += blocksize) {
	    for (int j = 0; j < xmax; j+= blocksize) {
		for (int ti = -searchrange; ti <= searchrange; ti++) {
		    for (int tj = -searchrange; tj <= searchrange; tj++) {
			if (method == 's')
			// Following line is CLIP overloaded operator ^ that
			// takes sum squared difference between two pictures
			 matches[ti+searchrange][tj+searchrange] =
			  in[yblock+i][xblock+j]^prev[yblock+i+ti][xblock+j+tj];
			else
			// Following line is CLIP overloaded operator | that
			// takes sum absolute difference between two pictures
			 matches[ti+searchrange][tj+searchrange] =
			  in[yblock+i][xblock+j]|prev[yblock+i+ti][xblock+j+tj];
			}
		    }
		int besti, bestj;
		int best = matches.min(besti, bestj);
		if (matches[searchrange][searchrange] == best)
			{
			// Prefer middle point
			besti = searchrange;
			bestj = searchrange;
			}
		besti -= searchrange;
		bestj -= searchrange;
		cout << i+blocksize/2 << " " << j+blocksize/2 << " ";
		cout << besti << " " << bestj << '\n';
		}
	    }
	return 1;
	}
